#include <string.h>

#include "dhcp.h"
#include "udp.h"

#include "timer.h"


/* current DHCP phase */
BYTE dhcp_phase;


/* application handler for incomming datagrams */
UDP_APP_HANDLER(dhcpc)
{
  unsigned int txbuf;

  int opts;
  unsigned char opt;
  unsigned char optl;

  BYTE opt_message=0;
  DWORD opt_server=0;
  DWORD opt_netmask=0;
  
  DWORD yourip;

  /* there should be at least bootp header and magic cookie for options */
  if (len<sizeof(struct bootp_header)+4) return;

  /* check operation, only reply allowed */
  if (netbuf_getc()!=BOOTP_OP_BOOTREPLY) return;

  netbuf_skip(3); /* htype, hlen, hops */

  /* check transaction ID */
  if (netbuf_getc()!=mymac[2]) return;
  if (netbuf_getc()!=mymac[3]) return;
  if (netbuf_getc()!=mymac[4]) return;
  if (netbuf_getc()!=mymac[5]) return;

  netbuf_skip(8); /* seconds, flags, clientip */

  netbuf_read(&yourip, 4);

  netbuf_skip(216); /* skip the rest rest up to the options */
  
  if (netbuf_getc()!=DHCP_OPT_MAGIC0) return;
  if (netbuf_getc()!=DHCP_OPT_MAGIC1) return;
  if (netbuf_getc()!=DHCP_OPT_MAGIC2) return;
  if (netbuf_getc()!=DHCP_OPT_MAGIC3) return;

  opts=len-sizeof(struct bootp_header)-4;

  /* parsing of DHCP options */
  while (opts>0) {
    opt=netbuf_getc();
    if (opt==DHCP_END) break;
    if (opt==DHCP_PAD) { opts--; continue; }
    optl=netbuf_getc();
    opts-=(optl+2);
    if (opts<0) return; /* incomplete packet? */
    switch (opt) {
      case DHCP_MESSAGE_TYPE:
        opt_message=netbuf_getc();
        break;
      case DHCP_SERVER_ID:
        netbuf_read(&opt_server,4);
        break;
      case DHCP_SUBNET_MASK:
        netbuf_read(&opt_netmask,4);
        break;
      default:
        netbuf_skip(optl);
        break;
    }
  }
  netbuf_close();
  
  switch (opt_message) {
    case DHCPOFFER:
      dhcp_phase=DHCPOFFER;

      /* send DHCPREQUEST message */
      if (!nic_tx_alloc(&txbuf)) return;
      netbuf_wr_seek(txbuf+UDP_RESERVED, NB_SEEK_SET);

      netbuf_putc(BOOTP_OP_BOOTREQUEST);
      netbuf_putc(BOOTP_HTYPE_ETH);
      netbuf_putc(6); /* hlen */
      netbuf_putc(0); /* hops */
      netbuf_write(mymac+2, 4); /* xid */
      netbuf_fill(0,2); /* seconds */
      netbuf_fill(0,2); /* flags */
      netbuf_fill(0,16); /* clientip, yourip, serverip, gatewayip */

      netbuf_write(mymac, 6); /* clientha */
      netbuf_fill(0,202); /* clientha, server_hostname, boot_filename */

      netbuf_putc(DHCP_OPT_MAGIC0);
      netbuf_putc(DHCP_OPT_MAGIC1);
      netbuf_putc(DHCP_OPT_MAGIC2);
      netbuf_putc(DHCP_OPT_MAGIC3);

      netbuf_putc(DHCP_MESSAGE_TYPE);
      netbuf_putc(1);
      netbuf_putc(DHCPREQUEST);

      netbuf_putc(DHCP_REQUESTED_IP);
      netbuf_putc(4);
      netbuf_write(&yourip, 4);

      netbuf_putc(DHCP_SERVER_ID);
      netbuf_putc(4);
      netbuf_write(&opt_server, 4);

      netbuf_putc(DHCP_END);

      len=netbuf_wr_tell()-txbuf-UDP_RESERVED;
      udp_out(IP_BROADCAST, HTONS(67), HTONS(68), txbuf, len);
      dhcp_phase=DHCPREQUEST;
      break;

    case DHCPACK:
      dhcp_phase=DHCPACK;
      myip=yourip;
      netmask=opt_netmask;
      udp_close_port(HTONS(68));
      break;
      
   case DHCPDECLINE:
   case DHCPNAK:
      dhcp_phase=opt_message;
      udp_close_port(HTONS(68));
      break;
  }
}


/* send a discover message and start listening for reply */
void dhcp_discover(void)
{
  unsigned int txbuf;
  unsigned int len;

  if (!nic_tx_alloc(&txbuf)) return;
  netbuf_wr_seek(txbuf+UDP_RESERVED, NB_SEEK_SET);

  netbuf_putc(BOOTP_OP_BOOTREQUEST);
  netbuf_putc(BOOTP_HTYPE_ETH);
  netbuf_putc(6); /* hlen */
  netbuf_putc(0); /* hops */
  netbuf_write(mymac+2, 4); /* xid */
  netbuf_fill(0,2); /* seconds */
  netbuf_fill(0,2); /* flags */
  netbuf_fill(0,16); /* clientip, yourip, serverip, gatewayip */

  netbuf_write(mymac, 6); /* clientha */
  netbuf_fill(0,202); /* clientha, server_hostname, boot_filename */

  netbuf_putc(DHCP_OPT_MAGIC0);
  netbuf_putc(DHCP_OPT_MAGIC1);
  netbuf_putc(DHCP_OPT_MAGIC2);
  netbuf_putc(DHCP_OPT_MAGIC3);

  netbuf_putc(DHCP_MESSAGE_TYPE);
  netbuf_putc(1);
  netbuf_putc(DHCPDISCOVER);
  netbuf_putc(DHCP_END);

  len=netbuf_wr_tell()-txbuf-UDP_RESERVED;

  udp_listen(HTONS(68),dhcpc);
  udp_out(IP_BROADCAST, HTONS(67), HTONS(68), txbuf, len);

  dhcp_phase=DHCPDISCOVER;
}


/* DHCP state machine */
void dhcpc_proc(void)
{
  static clock_t dhcp_lap;
  static unsigned char dhcp_timeout;

  /* check for final state, lease timeout not supported */
  if (dhcp_phase==DHCPACK || dhcp_phase==DHCPNAK || dhcp_phase==DHCPDECLINE)
    return;


  if (clock()-dhcp_lap>=1000) {

    dhcp_lap=clock();

    /* decrease timeout */
    if (dhcp_timeout) dhcp_timeout--;

    /* if network link is down, reset DHCP state and return */
    if (nic_link()==0) {
      dhcp_phase=0;
      return;
    }

    /* no message sent yet or there was a timeout */
    if (dhcp_phase==0 || dhcp_timeout==0) {
      dhcp_discover();
      dhcp_timeout=DHCP_TIMEOUT;
    }

  }
}
