#ifndef ipH
#define ipH

#include "platform.h"
#include "eth.h"


/* structure of IP header */
struct ip_header {
  /* BYTE ver_ihl; */
  unsigned ihl:4;
  unsigned ver:4;
  BYTE tos;
  WORD length;
  WORD identification;
  WORD fragment;
  BYTE ttl;
  BYTE protocol;
  WORD checksum;
  DWORD sourceip;
  DWORD destip;
  BYTE tail[];
};


/* minimal length of reserved buffer area when sending IP packet */
#define IP_RESERVED (ETH_RESERVED+sizeof(struct ip_header))


#define IP_FRAG_OFFSET   0x1fff
#define IP_FRAG_MORE     0x2000
#define IP_FRAG_DONT     0x4000
#define IP_FRAG_RESERVED 0x8000

#define IP_PROTO_ICMP 1
#define IP_PROTO_TCP  6
#define IP_PROTO_UDP 17

#define IP_BROADCAST 0xffffffffL


extern DWORD myip; /* local IP address in network byte order */
extern DWORD netmask; /* network mask */


/* creates IP address as DWORD in network byte order */
extern DWORD ip_addr(BYTE a1, BYTE a2, BYTE a3, BYTE a4);

/* print given IP address in human readable form to stdout */
extern void ip_print(DWORD ip);

/* configure local IP address according to given values */
extern void ip_config(DWORD ip, DWORD netmask);

/* calculates IP checksum of block of data */
extern WORD ip_checksum(WORD *wordp, unsigned int bytec);

/* input packet feed */
extern void ip_in(unsigned int len);

/* packet transmission */
extern void ip_out(DWORD destip, BYTE protocol, unsigned int txbuf, unsigned int len);

#endif
