#include <stdio.h>

#include "platform.h"

#include "uart.h"
#include "timer.h"
#include "eth.h"
#include "ip.h"
#include "udp.h"
#include "dhcp.h"
#include "nbns.h"

#include "pwm.h"
#include "adc.h"
#include "leddisp.h"


#if defined(_18F4520) | defined(_18F4620)
__CONFIG(1,0xF1FF);
__CONFIG(2,0xFEEE);
__CONFIG(3,0xFDFF);
__CONFIG(4,0xFFBB);
__CONFIG(5,0xFFFF);
__CONFIG(6,0xFFFF);
__CONFIG(7,0xFFFF);

#else /* 18f458 */
__CONFIG(1,0xF9FF);
__CONFIG(2,0xFEF2);
__CONFIG(3,0xFFFF);
__CONFIG(4,0xFFFB);
__CONFIG(5,0xFFFF);
__CONFIG(6,0xFFFF);
__CONFIG(7,0xFFFF);
#endif


void putch(unsigned char c)
{
  uart_putc(c);
}


/* respond to datagram on UPD port 2345 */
UDP_APP_HANDLER(pvkctl)
{
  unsigned int txbuf;
  unsigned int adval;
  
  if (len<5) return;

  disp_bargraph(netbuf_getc());
  disp_data[0]=netbuf_getc();
  disp_data[1]=netbuf_getc();
  disp_data[2]=netbuf_getc();
  disp_data[3]=netbuf_getc();
  netbuf_close();

  if (!nic_tx_alloc(&txbuf)) return;
  RB3=1;

  netbuf_wr_seek(UDP_RESERVED, NB_SEEK_CUR);
  
  adval=adc_sample(0);
  netbuf_write(&adval,2);

  adval=adc_sample(1);
  netbuf_write(&adval,2);

  adval=adc_sample(2);
  netbuf_write(&adval,2);

  adval=adc_sample(3);
  netbuf_write(&adval,2);

  netbuf_putc(buttons);
  
  udp_out(0,udph->sourceport,HTONS(2345),txbuf,9);
  
  RB3=0;
}


/* MAC address */
__EEPROM_DATA(0xA6,0x12,0x34,0x56,0x78,0x90, 0x00,0x00);

/* IP address and netmask */
__EEPROM_DATA(0x00,0x00,0x00,0x00, 0xff,0xff,0xff,0xff);


/* main application entry point */
void main(void)
{
  unsigned char dhcp_done=0;
  unsigned char ip_static=0;
  unsigned char i;

  CMCON=7; /* disable comparator */
  TRISB&=~0x08; /* output to LED */

  timer1_init();
  GIE=1;
  PEIE=1;

  pwm_init();
  adc_init();
  uart_init();
  
  disp_init();
  disp_hex(0xabcd);
  disp_bargraph(5);

  for (i=0; i<6; i++) mymac[i]=eeprom_read(i);
  nic_init(mymac);

  myip=ip_addr(eeprom_read(8),eeprom_read(9),eeprom_read(10),eeprom_read(11));
  netmask=ip_addr(eeprom_read(12),eeprom_read(13),eeprom_read(15),eeprom_read(15));
  if (myip) ip_static=1;

  nbns_start();
  udp_listen(HTONS(2345),pvkctl);
  
  for (;;) {
    eth_proc();

    if (!ip_static) {
      dhcpc_proc();
      if (!dhcp_done) {
        if (dhcp_phase==DHCPACK) {
          dhcp_done=1;
          printf("dhcp lease ");
          ip_print(myip);
          printf("/");
          ip_print(netmask);
          printf("\n");
          sound_ok();
        }
        else if (dhcp_phase==DHCPNAK || dhcp_phase==DHCPDECLINE) {
          dhcp_done=1;
          sound_failed();
        }
      }
    }
    
  }
}
